import importlib
import logging
import os
import subprocess
import sys
from argparse import ArgumentParser
from pathlib import Path

# DB Connectivity
from sqlalchemy import create_engine
from sqlalchemy.pool import NullPool

from .containers import Flavors, PostgreContainer
from .exceptions import UnknownFlavorException

# Logging Config
Path("./logs").mkdir(parents=True, exist_ok=True)
log_format = "[%(levelname)s] %(asctime)s - %(name)s - %(message)s"
logging.basicConfig(
    level=logging.DEBUG,
    format=log_format,
    datefmt="%y-%m-%dT%H:%M.%S",
    filename="./logs/gen.log",
    filemode="a",
)
formatter = logging.Formatter(log_format)


def main(argv=None):
    parser = ArgumentParser(
        description="Convert a SQLAlchemy declarative base into an Atlas HCL file."
    )
    parser.add_argument(
        "filepath",
        help="The relative path to the python module containing a SQLAlchemy declarative base.",
    )
    parser.add_argument(
        "--flavor",
        type=str,
        default=Flavors.POSTGRES.value,
        choices=[Flavors.POSTGRES.value],
        help="The flavor of the database HCL you want to generate.",
    )
    parser.add_argument(
        "--base",
        type=str,
        default="Base",
        help="The name of the declarative base object in the schema file.",
    )
    args = parser.parse_args(argv)
    path = args.filepath.replace("/", ".").replace(".py", "")

    sys.path.append(os.getcwd())
    mod = importlib.import_module(path)
    base = getattr(mod, args.base)

    try:
        if args.flavor == Flavors.POSTGRES.value:
            db = PostgreContainer()
        else:
            raise UnknownFlavorException(f"Flavor {args.flavor} not supported.")

        engine = create_engine(db.connection_string, poolclass=NullPool)
        base.metadata.create_all(engine)

        output = subprocess.run(
            ["atlas", "schema", "inspect", "--url", db.connection_string],
            shell=False,
            capture_output=True,
            text=True,
        )
        logging.info(output)

    finally:
        logging.debug("Completed generating HCL. Cleaning up.")
        try:
            db.container.kill()
            db.container.remove()
        except UnboundLocalError as err:
            logging.warning(
                "db container failed to initialize, nothing to kill: %s", err
            )
        logging.debug("Done cleaning up.")

    print(output.stdout, file=sys.stdout)


if __name__ == "__main__":
    main()



import enum
import logging
import secrets
import time

import docker


class Flavors(enum.Enum):
    POSTGRES = "postgres"


class PostgreContainer:
    """Postgres flavored container"""

    def __init__(
        self,
        image="postgres",
        tag="latest",
        port=5432,
        password=None,
        wait_for_ready=True,
    ):
        self.container = None

        self.db_image = image + ":" + tag
        self.db_port = port
        self.db_password = password or self._gen_password()

        self._init_db_container()
        if wait_for_ready:
            self._wait_for_db_ready()

    @property
    def connection_string(self):
        connection_string = f"postgresql://postgres:{self.db_password}@localhost:{self.db_port}/postgres?sslmode=disable"
        return connection_string

    def _wait_for_db_ready(self):
        ready_string = "database system is ready to accept connections"
        logging.debug("Waiting for the DB to be initialize...")
        for log in self.container.logs(stream=True):
            if ready_string in log.decode("ascii"):
                logging.debug("DB is ready with note: '%s'", ready_string)
                time.sleep(0.25)  # TODO: Invesitgate ready failure without this
                return

    def _gen_password(self):
        return secrets.token_urlsafe(8)

    def _init_db_container(self):
        container_name_hash = secrets.token_urlsafe(4).lower()
        container_name = f"dev-postgres-{container_name_hash}"
        logging.debug("Initializing DB container with name: '%s'", container_name)

        client = docker.from_env()
        self.container = client.containers.run(
            self.db_image,
            name=container_name,
            environment=[f"POSTGRES_PASSWORD={self.db_password}"],
            ports={"5432/tcp": self.db_port},
            detach=True,
        )
        logging.debug("Initializing DB container: %s", self.container)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sqlalchemy2atlas']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy>=1.4,<2.0', 'docker>=6.0,<7.0', 'psycopg2>=2.9,<3.0']

entry_points = \
{'console_scripts': ['sqlalchemy2atlas = '
                     'sqlalchemy2atlas.sqlalchemy2atlas:main']}

setup_kwargs = {
    'name': 'sqlalchemy2atlas',
    'version': '0.1.9',
    'description': 'Converts SQLALchemy Base Classes to AtlasGo HCL.',
    'long_description': '# Summary\nConverts [SQLAlchemy](https://www.sqlalchemy.org/) models to [AtlasGo](https://atlasgo.io/) HCL.\n\n# Quickstart\n<!-- Installation -->\n- Run `pip install sqlalchemy2atlas`\n<!-- Usage -->\n- Run `sqlalchemy2atlas -h`\n\n\n## Driver Support\n- [x] PostgreSQL\n    - [x] 14.x\n- [ ] Cloud Spanner\n- [ ] MySQL\n- [ ] SQLLite\n\n\n# Example\nExample of converting a SQLAlchemy module into AtlasGo HCL.\n\n```python\nfrom datetime import datetime\nfrom sqlalchemy.orm import declarative_base\nfrom sqlalchemy import (\n    Column,\n    Integer,\n    String,\n    DateTime,\n    CheckConstraint,\n    ForeignKey,\n)\n\nBase = declarative_base()\n\nclass User(Base):\n    __tablename__ = "user"\n\n    # Feat: Primary Key\n    id = Column(Integer, primary_key=True)\n\n    # Feat: Column Types\n    name = Column(String(30))\n    full_name = Column(String)\n    age = Column(Integer)\n\n    # Feat: Check Constraints\n    check_age_positive = CheckConstraint(age > 0)\n    check_age_reasonable = CheckConstraint(age < 200)\n\nclass Account(Base):\n    __tablename__ = "account"\n\n    # Feat: Primary Key\n    id = Column(Integer, primary_key=True)\n\n    # Feat: Column Types, Foreign Keys\n    user_id = Column(Integer, ForeignKey("user.id"), nullable=False)\n    account_created_at = Column(DateTime, nullable=False)\n\n```\n\nInto\n\n```go\ntable "account" {\n  schema = schema.public\n  column "id" {\n    null = false\n    type = serial\n  }\n  column "user_id" {\n    null = false\n    type = integer\n  }\n  column "account_created_at" {\n    null = false\n    type = timestamp\n  }\n  primary_key {\n    columns = [column.id]\n  }\n  foreign_key "account_user_id_fkey" {\n    columns     = [column.user_id]\n    ref_columns = [table.user.column.id]\n    on_update   = NO_ACTION\n    on_delete   = NO_ACTION\n  }\n}\ntable "user" {\n  schema = schema.public\n  column "id" {\n    null = false\n    type = serial\n  }\n  column "name" {\n    null = true\n    type = character_varying(30)\n  }\n  column "full_name" {\n    null = true\n    type = character_varying\n  }\n  column "age" {\n    null = true\n    type = integer\n  }\n  primary_key {\n    columns = [column.id]\n  }\n  check "user_age_check" {\n    expr = "(age > 0)"\n  }\n  check "user_age_check1" {\n    expr = "(age < 200)"\n  }\n}\nschema "public" {\n}\n```',
    'author': 'Fraser Isbester',
    'author_email': '28307321+Fraser-Isbester@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
